@@ -278,7 +278,7 @@ impl Params {
                     unpack_rest = Some(Stmt::For(ForStmt {
                         span,
                         init: Some(VarDeclOrExpr::VarDecl(VarDecl {
-                            kind: VarDeclKind::Let,
+                            kind: VarDeclKind::Var,
                             span,
                             decls: vec![
                                 // _len = arguments.length - i
