@@ -353,21 +353,12 @@ impl Spread {
                 .make_member(Ident::new(js_word!("concat"), DUMMY_SP))
                 .as_callee();
 
-            if buf[0].spread.is_none() {
-                return Expr::Call(CallExpr {
-                    span,
-                    callee,
-                    args: buf,
-                    type_args: Default::default(),
-                });
-            } else {
-                return Expr::Call(CallExpr {
-                    span,
-                    callee,
-                    args: buf,
-                    type_args: Default::default(),
-                });
-            }
+            return Expr::Call(CallExpr {
+                span,
+                callee,
+                args: buf,
+                type_args: Default::default(),
+            });
         }
 
         Expr::Call(CallExpr {
